#include "denkou.h"

int f_v = 0;

void printUsageAndExit(void)
{
  char **pp;
  static char *usage[] = {
    "Usage1: denkou [options] text_file ...",
    "Usage2: denkou [options] -c message ...",
    "Usage3: denkou [options] -s                # input from stdin",
    "Usage4: denkou [options] -w text_file ...  # write into eeprom",
    "  options",
    "    -a ank_font       set ANK font file(pcf)",
    "    -b bright         set LED brightness (0-10)",
    "    -d speed          set scroll speed (1=fastest)",                
    "    -e sjis|euc       input kanji code",
    "    -f rcfile         read rc file", 
    "    -F fontdir        set font direcotry",
    "    -h                show this",
    "    -k kanji_font     set Kanji font file(pcf)",
    "    -l                loop message",
    "    -n                no scroll",
    "    -M number         set number of LED module",
    "    -q                quit mode",
    "    -Q                do not read rcfile ",
    "    -R                set LED reverse",
    "    -t tty            set tty device",
    "    -T 0|1|2          vertical mode (0:none 1:kanji 2:kanji+ank)",
    "    -v                verbose mode",
    "    -V                show version and exit",
    "",
    NULL,
  };

  for(pp=usage;*pp!=NULL;pp++)
    fprintf(stderr,"%s\n",*pp);
  exit(1);
}

void dcont_init(dcont_t *dcont)
{
  dcont->cmdline_message_mode = 0;
  dcont->args = NULL;

  sio_init(&dcont->scont);

  dcont->font_dir   = NULL;
  dcont->font1_name = "8x16rk.pcf";
  dcont->font2_name = "jiskan16.pcf";

  dcont->encoding  = Enc_SJIS;
  dcont->buf_left  = 16;
  dcont->div       = 4;
  dcont->no_scroll = 0;
  dcont->b_stdin   = 0;
  dcont->char0     = 0;
  dcont->b_loop    = 0;
  dcont->b_rom_write = 0;
  dcont->no_out      = 0;
  dcont->nol         = 0;
  dcont->b_no_rcfile = 0;
  dcont->b_vertical  = 0;
  dcont->b_vertical_ank = 0;
  dcont->b_led_reverse = 0;
  dcont->bright = 10;
  dcont->cmodule = 1;
}

void process_command_line(dcont_t *dcont, int argc, char *argv[])
{
  extern char *optarg;
  extern int optind;
  int c;

  while(1){
    c = getopt(argc, argv, "a:b:cd:e:f:F:hk:lM:nvqRst:T:Vw");
    if(c == -1)
      break;
    switch(c){
    case 'a':
      dcont->font1_name = optarg;
      break;
    case 'b':
      if(sscanf(optarg,"%d",&dcont->bright)<1)
	nabort("Bad number \"%s\"", optarg);
      break;
    case 'c':
      dcont->cmdline_message_mode = 1;
      break;
    case 'd':
      if(sscanf(optarg,"%d",&dcont->div)<1)
	nabort("Bad number \"%s\"",optarg);
      break;
    case 'e':
      if(strcasecmp(optarg,"sjis")==0)
	dcont->encoding = Enc_SJIS;
      else if(strcasecmp(optarg,"euc")==0)
	dcont->encoding = Enc_EUC;
      else nabort("bad -e argment \"%s\", shoud be sjis or euc\n",optarg);
      break;
    case 'f':
      if(!read_rcfile(dcont, ".", optarg))
	nabort("%s not found",optarg);
      break;
    case 'F':
      dcont->font_dir = ntStringNew(optarg);
      break;
    case 'h':
      printUsageAndExit();
      break;
    case 'k':
      dcont->font2_name = optarg;
      break;
    case 'l':
      dcont->b_loop = 1;
      break;
    case 'M':
      if(sscanf(optarg,"%d",&dcont->cmodule)<1)
	nabort("Bad number \"%s\"",optarg);
      break;
    case 'n':
      dcont->no_scroll = 1;
      break;
    case 'R':
      dcont->b_led_reverse = 1;
      break;
    case 'v':
      f_v++;
      dcont->scont.f_v++; 
      break;
    case 'V':
      printf("denkou %s\n",ver_str);
      exit(0);

    case 'q':
      f_v--;
      break;
    case 's':
      dcont->b_stdin = 1;
      break;
    case 't':
      dcont->scont.tty_name = optarg;
      break;
    case 'T':
      if(strcmp(optarg,"0")==0)
	dcont->b_vertical = dcont->b_vertical_ank = 0;
      else if(strcmp(optarg,"1")==0)
	dcont->b_vertical = 1, dcont->b_vertical_ank = 0;
      else if(strcmp(optarg,"2")==0)
	dcont->b_vertical = dcont->b_vertical_ank = 1;
      else
	nabort("vertical mode %s not defined. it should be 0|1|2", optarg);
      break;
    case 'w':
      dcont->b_rom_write = 1;
      break;
    case '?':
    default:
      printUsageAndExit();
      break;
    }
  }
  if(optind >= argc && !dcont->b_stdin)
    printUsageAndExit();
  dcont->args = argv+optind;
}

int main(int argc, char *argv[])
{
  dcont_t dcont;
  char **pp;
  int c;
  sio_t *scont = &dcont.scont;

  dcont_init(&dcont);
  if(argc > 1 && strcmp(argv[1],"-Q")==0)
    dcont.b_no_rcfile = 1;

  if(!dcont.b_no_rcfile)
    read_rcfiles(&dcont);
  
  process_command_line(&dcont, argc, argv);
  
  if(f_v) printf("denkou %s\n",ver_str);

  dcont.font1 = guFontNew(dcont.font_dir, dcont.font1_name);
  dcont.font2 = guFontNew(dcont.font_dir, dcont.font2_name);
  
  sio_open(scont);
  sio_cmd(scont, "echo 0");
  sio_cmd(scont, "set scmode_div %d",    dcont.div);
  sio_cmd(scont, "lm_bright %d",         dcont.bright);
  sio_cmd(scont, "set cmodule %d\n",     dcont.cmodule);
  sio_cmd(scont, "set led_reverse %d\n", dcont.b_led_reverse);
  
  if(dcont.cmdline_message_mode){
    do {
      dk_open(&dcont);
      for(pp=dcont.args; *pp!=NULL; pp++){
	dk_puts(&dcont, *pp);
	if(pp[1]!=NULL) dk_puts(&dcont, " ");
      }
      dk_close(&dcont);
      dk_wait_scroll_end(&dcont);
    } while(dcont.b_loop);
  }
  else if(dcont.b_stdin){
    do {
      dk_open(&dcont);
      while((c = getchar())!=EOF){
	dk_putc(&dcont,c);
	if(dcont.no_scroll && dcont.buf_left == 0) break;
      }
      dk_close(&dcont);
      dk_wait_scroll_end(&dcont);
    } while(dcont.b_loop);
  }
  else if(dcont.b_rom_write){
    write_rom(&dcont);
  }
  else {
    do {
      dk_open(&dcont);
      for(pp=dcont.args; *pp!=NULL; pp++){
	FILE *ifp = fopen(*pp,"r");
	if(ifp == NULL) fprintf(stderr,"file \"%s\" not found.\n",*pp);
	else {
	  dcont.char0 = 0;
	  while((c = getc(ifp))!=EOF){
	    dk_putc(&dcont,c);
	    if(dcont.no_scroll && dcont.buf_left < 16) break;
	  }
	  fclose(ifp);
	}
      }
      dk_close(&dcont);
      dk_wait_scroll_end(&dcont);               /* $B%9%/%m!<%k=*N;$rBT$D(B */
    } while(dcont.b_loop);
  }
  sio_close(scont);
  return 0;
}

