#ifndef _guFont_h
#define _guFont_h

typedef long  int32;
typedef short int16;

typedef unsigned long  uint32;
typedef unsigned short uint16;
typedef unsigned char  uint8,uchar,byte8;

typedef struct _pcf_toc{
  char    version[4];
  uint32  nTables;
  struct _pcf_toc_table {
    uint32 type;
    uint32 format;
    uint32 size;
    uint32 offset;
  } tables[0];
} Pcf_Toc;

#define   PCF_PROPERTIES        (1 << 0)
#define   PCF_ACCELERATORS      (1 << 1)
#define   PCF_METRICS           (1 << 2)
#define   PCF_BITMAPS           (1 << 3)
#define   PCF_INC_METRICS       (1 << 4)
#define   PCF_BDF_ENCODINGS     (1 << 5)
#define   PCF_SWIDTHS           (1 << 6)
#define   PCF_GLYPH_NAMES       (1 << 7)
#define   PCF_BDF_ACCELERATORS  (1 << 8)

typedef struct _pcf_encodings {
  uint32 format;
  int16  firstCol;
  int16  lastCol;
  int16  firstRow;
  int16  lastRow;
  int16  defaultCh;
  int16  encodings[0];
} Pcf_Encodings;

#define Is_CMetrics(f)  (((f)&0xffffff00)==0x100)

typedef struct _pcf_meric_t {
  uint32 format;
  int32  nMetrics;
  struct _pcf_metric_table {
    int16  lbear;
    int16  rbear;
    int16  width; 
    int16  ascent;
    int16  descent;
    int16  attributes;
  } metrics[0];
} Pcf_Metric_Sec;

typedef struct _pcf_metric_table FontMetric;

typedef struct _pcf_cmeric_t {
  uint32 format;
  int16  nMetrics;
  struct _pcf_cmetric_table {
    uint8  lbear;
    uint8  rbear;
    uint8  width;
    uint8  ascent;
    uint8  descent;
  } cmetrics[0];
} Pcf_CMetric_Sec;

typedef struct _pcf_bitmaps {
  uint32 format;
  int32  nBitmaps;
  uint32 bitmapOffsets[0];
} Pcf_Bitmaps_Sec;

typedef struct _guFont {
  char    *filename;
  int     bOpened;
  int     fontFd;
  uchar   *map;
  off_t   filesize;
  Pcf_Toc          *pcf_toc; 
  Pcf_Encodings    *encode;
  Pcf_Metric_Sec   *metric;
  Pcf_CMetric_Sec  *cmetric;
  Pcf_Bitmaps_Sec  *bitmaps;
  byte8 *bitmap_base;
  uint8 bytePerData;
  uint8 lineAlign;
  uint8 bMsbFirst;
  uint8 bBigEndian;
} GuFont;

typedef struct _guGlyph{
  int   rbear;
  int   lbear;
  int   ascent;
  int   descent;
  int   width;
  int   bpl;
  byte8 *bitmap;
} GuGlyph;

/* prototypes */

GuFont *guFontNew(char *dirName, char *filename);
void guFontOpen(GuFont *font);
void guFontClose(GuFont *font);
void guFontDump(GuFont *font,unsigned long code);
int guFontEncode(GuFont *font, unsigned long code);
void guFontMetric(GuFont *font,int n, FontMetric *pFm);
void guFontBitmap(GuFont *font,int idx,byte8 **pData);
void guFontGlyph(GuFont *font,unsigned long code,GuGlyph *glyph);
void guGlyphDump(GuGlyph *glyph);
int dk_get_glyph_bitmap(GuGlyph *glyph,unsigned short *buf,int buflen);

#endif /* _guFont_h */
