#ifdef  __CYGWIN__
#include <windows.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

#include "sio.h"
#include "nabort.h"

void sio_open(sio_t *scont)
{
  int h;
  speed_t s;
  struct termios ts;
  
  scont->tty_h = h = open(scont->tty_name, O_RDWR|O_NONBLOCK);
  if(h < 0) nabort("tty port \"%s\" not found.\n",scont->tty_name);

  if(tcgetattr(h, &ts)) nabort("tcgetattr failed.\n");
#if 0
  cfmakeraw(&ts);
  ts.c_cflag |=  CSTOPB;
#endif
  
  switch(scont->tty_baud){
  case  9600: s = B9600;  break;
  case 19200: s = B19200; break;
  default: nabort("baudrate %d not supported yet.\n",scont->tty_baud);
  }
  cfsetospeed(&ts, s);
  cfsetispeed(&ts, s);
  
  if(tcsetattr(h, TCSADRAIN, &ts)) nabort("tcsetattr failed.\n");
}

void sio_close(sio_t *scont)
{
  scont->tty_h = -1;
}

int sio_write(sio_t *scont,const char *buf, int len)
{
  return write(scont->tty_h, buf, len);
}

int sio_read(sio_t *scont, char *buf, int len)
{
  int n = read(scont->tty_h, buf, len);
  if(n < 0 && errno == EAGAIN) n = 0;
  return n;
}

void sio_rts_set(sio_t *scont, int val)
{
  int flags;

  if(ioctl(scont->tty_h, TIOCMGET, &flags) < 0) {
    perror("ioctl");
    exit(1);
  }
  if(val) flags |=  TIOCM_RTS;
  else    flags &= ~TIOCM_RTS;
  if(ioctl(scont->tty_h, TIOCMSET, &flags) < 0) {
    perror("ioctl");
    exit(1);
  }
}

void sio_dtr_set(sio_t *scont, int val)
{
  int flags;

  if(ioctl(scont->tty_h, TIOCMGET, &flags) < 0) {
    perror("ioctl");
    exit(1);
  }
  if(val) flags |=  TIOCM_DTR;
  else    flags &= ~TIOCM_DTR;
  if(ioctl(scont->tty_h, TIOCMSET, &flags) < 0) {
    perror("ioctl");
    exit(1);
  }
}

/**
 *   SIO͂ BLOCK[hݒ
 */
void sio_block_set(sio_t *scont,int bOn)
{
  int flag;
  
  if(fcntl(scont->tty_h, F_GETFL, &flag) < 0){
    perror("sio_block");
    exit(1);
  }
  if(bOn) flag &= ~O_NONBLOCK;
  else    flag |=  O_NONBLOCK;
  if(fcntl(scont->tty_h, F_SETFL, &flag)<0){
    perror("sio_block");
    exit(1);
  }
}
