#include <windows.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>

#include "sio.h"
#include "nabort.h"

void sio_open(sio_t *scont)
{
  DCB dcb;
  COMMTIMEOUTS timeouts;

  scont->comHandle = CreateFile(scont->tty_name,
				GENERIC_READ|GENERIC_WRITE, 0, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
  if(scont->comHandle == INVALID_HANDLE_VALUE)
    nabort("tty port \"%s\" not found.\n",scont->tty_name);

  if(!GetCommState(scont->comHandle, &dcb))
    nabort("error in GetCommState");
	
  dcb.BaudRate = scont->tty_baud;
  dcb.ByteSize = 8;
  dcb.Parity   = NOPARITY;
  dcb.StopBits = TWOSTOPBITS;

  if(!SetCommState(scont->comHandle, &dcb))
    nabort ("error in SetCommState");

  timeouts.ReadIntervalTimeout         = MAXDWORD;
  timeouts.ReadTotalTimeoutMultiplier  = 0;
  timeouts.ReadTotalTimeoutConstant    = 0;
  timeouts.WriteTotalTimeoutMultiplier = 0;
  timeouts.WriteTotalTimeoutConstant   = 0;
  SetCommTimeouts(scont->comHandle, &timeouts);

  EscapeCommFunction(scont->comHandle, SETDTR);
  EscapeCommFunction(scont->comHandle, SETRTS);
}

void sio_close(sio_t *scont)
{
  if(scont->comHandle != INVALID_HANDLE_VALUE){
    CloseHandle(scont->comHandle);
    scont->comHandle = INVALID_HANDLE_VALUE;
  }
}

int sio_write(sio_t *scont, const char *buf, int len)
{
  DWORD n;
  if(!WriteFile(scont->comHandle, buf, len, &n, 0))
    return -1;
  return n;
}

int sio_read(sio_t *scont, char *buf, int len)
{
  DWORD n;
  if(!ReadFile(scont->comHandle, buf, len, &n, 0))
    return -1;
  return n;
}

void sio_rts_set(sio_t *scont, int val)
{
  EscapeCommFunction(scont->comHandle, val ? SETRTS : CLRRTS);
}

void sio_dtr_set(sio_t *scont, int val)
{
  EscapeCommFunction(scont->comHandle, val ? SETDTR : CLRDTR);
}


